package org.rcisoft.business.device.assets.service.impl;

import org.rcisoft.business.device.assets.dao.BusDeviceRepository;
import org.rcisoft.business.device.assets.service.BusDeviceService;
import org.rcisoft.business.device.assets.vo.DeviceAssetStatisticVo;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.business.manage.entity.BusDevicetpParam;
import org.rcisoft.business.manage.entity.BusLibraryParam;
import org.rcisoft.business.manage.service.BusDeviceParamService;
import org.rcisoft.business.manage.service.BusDevicetpParamService;
import org.rcisoft.business.manage.service.BusLibraryParamService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.device.assets.entity.BusDevice;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.text.SimpleDateFormat;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import lombok.extern.slf4j.Slf4j;
import tk.mybatis.mapper.entity.Example;

/**
 * Created by  on 2018-4-12 15:33:23.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusDeviceServiceImpl implements BusDeviceService {

    @Autowired
    private BusDeviceRepository busDeviceRepository;
    @Autowired
    private BusDeviceParamService busDeviceParamService;
    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;
    @Autowired
    private BusDevicetpParamService busDevicetpParamService;


    /**
    * 保存 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusDevice busDevice){
        int line = 1;
        String message = "";
        //增加操作
        busDevice.setDevId(UUID.randomUUID().toString().replace("-",""));
        if(busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum())!=null && busDeviceRepository.queryDeviceByDevNum(busDevice.getDevNum()).size()>0){
            line = 0;
            message = "设备编号已存在，新增失败";
        }else{
            line = busDeviceRepository.insertSelective(busDevice);
        }

        return new PersistModel(line,message);
    }


    /**
    * 修改 busDevice
    * @param busDevice
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusDevice busDevice){
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",busDevice.getDevId());
        if(busDevice.getDevId()!=null && !busDevice.getDevId().equals("")){
            line = busDeviceRepository.updateByExampleSelective(busDevice,example);
        }else{
            line = 0;
            message = "设备ID为空，更新失败";
        }

        return new PersistModel(line,message);
    }

    /**
     * 删除
     * @param devid
     * @return
     */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devid) {
        int line = 0;
        String message = "";
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",devid);
        Map<String,Object> map = new HashMap<>();
        map.put("dev_id",devid);
        if(devid != null && !devid.equals("")){
            if(busDeviceRepository.queryOriginalByDev(map)==null || busDeviceRepository.queryOriginalByDev(map).size()<1){
                line = busDeviceRepository.deleteByExample(example);
            }else{
                message = "设备存在关联数据，删除失败";
            }
        }else{
            message = "设备ID为空，删除失败";
        }

        return new PersistModel(line,message);
    }

    /**
    * 根据id查询 busDevice
    * @param id
    * @return
    */
    @Override
    public List<BusDevice> findById(String id){
        Example example = new Example(BusDevice.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devId",id);
        return busDeviceRepository.selectByExample(example);
    }

    /**
    * 分页查询 busDevice
    * @param busDevice
    * @return
    */
    @Override
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusDevice> paginationUtility,
                                               BusDevice busDevice){
        //避免查出所有设备
        if(busDevice.getProId()!=null && !busDevice.getProId().equals("")){
            List<Map<String,Object>> list = busDeviceRepository.queryBusDevices(busDevice);
            for(Map<String,Object> tmap : list){
                tmap.put("paramList",busDeviceParamService.findByDev(tmap.get("DEV_NUM").toString(),tmap.get("PRO_ID").toString()));
                tmap.put("libraryparam",busLibraryParamRepository.queryLibraryParamByDev(tmap.get("DEV_NUM").toString()));
                BusDevicetpParam busDevicetpParam = new BusDevicetpParam();
                busDevicetpParam.setProId(tmap.get("PRO_ID").toString());
                busDevicetpParam.setDevTpId(tmap.get("DEV_TP_ID").toString());
                tmap.put("deviceTpParam",busDevicetpParamService.queryBusDevicetpParam(busDevicetpParam));
            }
            return list;
        }
        return null;
    }
    /*
        通过项目id与子系统id获取子系统中的所有设备
        @param sysId  子系统id
        @param proId  项目id
         */
    @Override
    public List<DeviceAssetStatisticVo> getDevicesBySysId(String sysId, String proId) {
        return busDeviceRepository.getBusDevicesBySysId(sysId,proId);
    }

    @Override
    public String getProNameByProId(String proId) {
        return busDeviceRepository.getProNameByProId(proId);
    }

    @Override
    public String getSysNameBySysId(String sysId) {
        return busDeviceRepository.getSysNameBySysId(sysId);
    }
}
