package org.rcisoft.business.evaluate.analysis.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.evaluate.analysis.service.AnalysisService;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;

/**
 * Created by JiChao on 2018/5/2.
 * 评估--分析报告
 */
@RestController
@RequestMapping("analysis")
public class AnalysisController {

    @Autowired
    private AnalysisService analysisServiceImpl;

    @ApiOperation(value="上传文件", notes="上传文件")
    @ApiImplicitParams({@ApiImplicitParam(name = "proId", value = "项目主键", required = true, dataType = "字符串"),
            @ApiImplicitParam(name = "year", value = "年", required = true, dataType = "数字"),
            @ApiImplicitParam(name = "month", value = "月", required = true, dataType = "数字")
    })
    @RequestMapping("/upload")
    public Result upload(@RequestParam String proId, @RequestParam Integer year, @RequestParam Integer month, @RequestParam MultipartFile file) {
        Integer result = analysisServiceImpl.upload(proId, year, month, file);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

    @ApiOperation(value="下载文件", notes="下载文件")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "文件的主键", required = true, dataType = "数字")})
    @RequestMapping("/download/{id:\\w+}")
    public void download(HttpServletResponse response, @PathVariable Integer id) {
        analysisServiceImpl.download(response, id);
    }

    @ApiOperation(value="查询列表", notes="根据年份、项目主键查询已经存在的文件列表，最多返回12条记录")
    @ApiImplicitParams({@ApiImplicitParam(name = "proId", value = "项目主键", required = true, dataType = "字符串"),
            @ApiImplicitParam(name = "year", value = "年", required = true, dataType = "数字")
    })
    @RequestMapping("/analysisFile")
    public Result analysisFile(@RequestParam String proId, @RequestParam Integer year) {
        return Result.builder(new PersistModel(1), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, analysisServiceImpl.analysisFile(proId, year));
    }

    @ApiOperation(value="删除文件", notes="删除文件")
    @ApiImplicitParams({@ApiImplicitParam(name = "id", value = "文件的主键", required = true, dataType = "数字")})
    @RequestMapping("/delete/{id:\\w+}")
    public Result delete(@PathVariable Integer id) {
        Integer result = analysisServiceImpl.delete(id);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }

}
