package org.rcisoft.business.evaluate.analysis.service.impl;

import org.apache.commons.io.FileUtils;
import org.rcisoft.business.evaluate.analysis.dao.BusReportRepository;
import org.rcisoft.business.evaluate.analysis.entity.BusReport;
import org.rcisoft.business.evaluate.analysis.service.AnalysisService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by JiChao on 2018/5/3.
 */
@Service
public class AnalysisServiceImpl implements AnalysisService {

    @Value("${filepath.analysis}")
    private String filepath;

    @Autowired
    private BusReportRepository busReportRepository;

    @Transactional
    @Override
    public Integer upload(String proId, Integer year, Integer month, MultipartFile file) {
        int result = 0;
        BusReport busReport = new BusReport(null, proId, year, month, null);
        String fileName = file.getOriginalFilename();
        String suffixName = fileName.substring(fileName.lastIndexOf("."));
        //文件路径
        String savePath = filepath + proId + File.separator + year + File.separator + month + suffixName;
        File saveFile = new File(savePath);
        //如果存在，删除文件，删除数据库记录
        if (saveFile.exists()) {
            saveFile.delete();
            busReportRepository.delete(busReport);
        }
        //保存
        busReport.setPath(savePath);
        try {
            FileUtils.copyInputStreamToFile(file.getInputStream(), saveFile);
            result = busReportRepository.insert(busReport);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return result;
    }

    @Override
    public void download(HttpServletResponse response, Integer id) {
        BusReport busReport = new BusReport();
        busReport.setId(id);
        busReport = busReportRepository.selectOne(busReport);
        String path = busReport.getPath();
        String filename = busReport.getProId() + "-" + busReport.getYear() + "-" + busReport.getMonth() + path.substring(path.lastIndexOf("."));
        try(OutputStream outputStream = response.getOutputStream()) {
            //获取文件字节
            byte[] bytes = FileUtils.readFileToByteArray(new File(path));
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + URLEncoder.encode(filename, "iso-8859-1"));
            outputStream.write(bytes);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public List<BusReport> analysisFile(String proId, Integer year) {
        //返回值，12个
        List<BusReport> resultList = new ArrayList<>();
        for (int i = 0; i < 12; i++) {
            resultList.add(new BusReport(-1, null, null, i + 1, null));
        }
        BusReport busReport = new BusReport();
        busReport.setProId(proId);
        busReport.setYear(year);
        List<BusReport> list = busReportRepository.select(busReport);
        list.forEach(b -> {
            Integer month = b.getMonth();
            BusReport br = resultList.get(month - 1);
            br.setId(b.getId());
            br.setMonth(month);
        });
        return resultList;
    }

    @Transactional
    @Override
    public Integer delete(Integer id) {
        BusReport busReport = new BusReport();
        busReport.setId(id);
        busReport = busReportRepository.selectOne(busReport);
        String path = busReport.getPath();
        File file = new File(path);
        if (file.exists()) {
            file.delete();
        }
        return busReportRepository.delete(busReport);
    }
}
