package org.rcisoft.business.mainte.adaptive.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.rcisoft.business.mainte.adaptive.dao.BusParamReferRepository;
import org.rcisoft.business.mainte.adaptive.dao.BusTemperatureRepository;
import org.rcisoft.business.mainte.adaptive.dao.TotalSensorRepository;
import org.rcisoft.business.mainte.adaptive.entity.BusParamRefer;
import org.rcisoft.business.mainte.adaptive.entity.BusTemperature;
import org.rcisoft.business.mainte.adaptive.entity.TotalSensor;
import org.rcisoft.business.mainte.adaptive.proenum.ProEnum;
import org.rcisoft.business.mainte.adaptive.service.AdaptiveService;
import org.rcisoft.business.mainte.adaptive.vo.BuildingAdaptation;
import org.rcisoft.business.mainte.adaptive.vo.ClimateAdaptation;
import org.rcisoft.business.mainte.adaptive.vo.Params;
import org.rcisoft.business.overview.dao.BusProjectRepository;
import org.rcisoft.business.overview.entity.BusProject;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.net.URL;
import java.net.URLConnection;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * Created by JiChao on 2018/4/28.
 */
@Service
public class AdaptiveServiceImpl implements AdaptiveService {

    @Autowired
    private TotalSensorRepository totalSensorRepository;
    @Autowired
    private BusParamReferRepository busParamReferRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private BusTemperatureRepository busTemperatureRepository;

    /**
     * 根据项目id得到项目的城市code
     * @param proId
     * @return
     */
    private String getCode(String proId) {
        BusProject b = new BusProject();
        b.setProId(proId);
        BusProject busProject = busProjectRepository.selectOne(b);
        return busProject.getCode();
    }

    @Override
    public List<Object> buildingList(Params params) {
        Calendar cal = Calendar.getInstance();
        //查询建筑负荷：供水温度，回水温度，水流量
        String[] code_array = new String[]{"", "", ""};
        List<BusParamRefer> busParamRefer = busParamReferRepository.queryOtherParam(params);
        busParamRefer.forEach(b -> {
            if(StringUtils.equals(b.getOwnParam(), ProEnum.gswd.toString())) code_array[0] = b.getOtherParam();
            if(StringUtils.equals(b.getOwnParam(), ProEnum.hswd.toString())) code_array[1] = b.getOtherParam();
            if(StringUtils.equals(b.getOwnParam(), ProEnum.sll.toString())) code_array[2] = b.getOtherParam();
        });
        //从sensor表查询对应日期的所有记录
        List<TotalSensor> totalSensorList = totalSensorRepository.queryTotalSensorList(params);
        List<Object> buildingList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        totalSensorList.forEach(totalSensor -> {
            cal.setTime(totalSensor.getTm());
            //整点数据
            if (cal.get(Calendar.MINUTE) == 0) {
                //得到小时
                int hour = cal.get(Calendar.HOUR_OF_DAY);
                //得到json对象
                JSONObject json = JSONObject.parseObject(totalSensor.getSensorJson());
                //供水温度
                BigDecimal gswd = (BigDecimal) json.get(code_array[0]);
                //回水温度
                BigDecimal hswd = (BigDecimal) json.get(code_array[1]);
                //水流量
                BigDecimal sll = (BigDecimal) json.get(code_array[2]);
                //公式计算
                Float building = Math.abs(gswd.subtract(hswd).multiply(sll).multiply(new BigDecimal(4.12)).divide(new BigDecimal(3.6), 1, BigDecimal.ROUND_HALF_UP).floatValue());
                buildingList.set(hour, building);
            }
        });
        return buildingList;
    }

    @Override
    public ClimateAdaptation climateAdaptation(String proId, String year, String month, String day) {
        Calendar cal = Calendar.getInstance();
        //查询实际供水温度
        Params params = new Params(proId, year + "-" + month + "-" + day, null);
        //1.从sensor表查询对应日期的所有记录
        List<TotalSensor> totalSensorList = totalSensorRepository.queryTotalSensorList(params);
        //2.从bus_param_refer表查询“实际供水温度”对应的code
        String[] code_array = new String[]{""};
        String gswd_code = ProEnum.gswd.toString();
        List<BusParamRefer> busParamReferList = busParamReferRepository.queryOtherParam(params);
        Iterator<BusParamRefer> it = busParamReferList.iterator();
        while (it.hasNext()) {
            BusParamRefer busParamRefer = it.next();
            if(StringUtils.equals(gswd_code, busParamRefer.getOwnParam())){
                code_array[0] = busParamRefer.getOtherParam();
                break;
            }
        }
        //3.取出数据中的整点记录，根据温度的code将数据取出，放入结果中
        List<Object> realList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        //循环
        totalSensorList.forEach(totalSensor -> {
            cal.setTime(totalSensor.getTm());
            //整点数据
            if (cal.get(Calendar.MINUTE) == 0) {
                //得到小时
                int hour = cal.get(Calendar.HOUR_OF_DAY);
                //得到json对象
                JSONObject json = JSONObject.parseObject(totalSensor.getSensorJson());
                //得到供水温度
                Object gswd = json.get(code_array[0]);
                realList.set(hour, gswd);
            }
        });
        //查询最优供水温度
        //1.根据proId查询项目的城市code
        String code = this.getCode(proId);
        //2.根据城市code查询对应日期的室外温度
        params.setCode(code);
        List<BusTemperature> busTemperaturesList = busTemperatureRepository.queryBusTemperatureList(params);
        //3.根据公式算出最优供水温度，不需要计算的数值取原来的数据
        List<Object> optimumList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        busTemperaturesList.forEach(busTemperature -> {
            cal.setTime(busTemperature.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            //得到时间
            BigDecimal temperature = busTemperature.getTemperature();
            //计算公式
            BigDecimal optimum = new BigDecimal(0);
            if (temperature.compareTo(new BigDecimal(35)) > 0) {//大于35度
                optimum = new BigDecimal(0.4).multiply(temperature).subtract(new BigDecimal(6));
            } else if (temperature.compareTo(new BigDecimal(26)) > 0 && temperature.compareTo(new BigDecimal(35)) < 1) {//大于26度，小于等于35度
                optimum = new BigDecimal(94).subtract(new BigDecimal(2).multiply(temperature)).divide(new BigDecimal(3));
            } else if (temperature.compareTo(new BigDecimal(0)) < 1) {//小于0度
                optimum = new BigDecimal(0).subtract(temperature).add(new BigDecimal(40));
            } else {//大于0度，小于等于26度
                optimum = temperature;
            }
            //放入结果集
            optimumList.set(hour, optimum);
        });
        //最后返回结果
        return new ClimateAdaptation(realList, optimumList);
    }

    @Override
    public BuildingAdaptation buildingAdaptation(String proId, String year, String month, String day) {
        Calendar cal = Calendar.getInstance();
        Params params = new Params(proId, year + "-" + month + "-" + day, this.getCode(proId));
        //查询室外温度
        List<BusTemperature> busTemperaturesList = busTemperatureRepository.queryBusTemperatureList(params);
        List<Object> temperatureList = Arrays.asList(new Object[]{0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0});
        busTemperaturesList.forEach(busTemperature -> {
            cal.setTime(busTemperature.getTm());
            int hour = cal.get(Calendar.HOUR_OF_DAY);
            BigDecimal temperature = busTemperature.getTemperature();
            temperatureList.set(hour, temperature);
        });
        List<Object> buildingList = this.buildingList(params);
        return new BuildingAdaptation(buildingList, temperatureList);
    }

    @Transactional
    @Override
    public PersistModel temperature() {
        List<BusTemperature> saveList = new ArrayList<>();
        //1.查询所有项目的城市code，去重复
        List<String> cityCodeList = busProjectRepository.allCityCode();
        //2.根据项目code，查询24小时的温度json
        cityCodeList.forEach(s -> {
            saveList.addAll(this.fromWeather(s));
        });
        //3.将所有的结果插入bus_temperature表(批量)
        Integer i = busTemperatureRepository.batchSaveBusTemperature(saveList);
        return new PersistModel(i);
    }

    private List<BusTemperature> fromWeather(String code) {
        List<BusTemperature> resultList = new ArrayList<>();
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String apiUrl = String.format("http://tj.nineton.cn/Heart/index/future24h/?city=%s", code);
        URL url = null;
        URLConnection open = null;
        InputStream input = null;
        try {
            //开始请求
            url = new URL(apiUrl);
            open = url.openConnection();
            input = open.getInputStream();
            //变成string
            String result = IOUtils.toString(input);
            JSONObject jsonResult = JSONObject.parseObject(result);
            JSONArray jsonArray = (JSONArray) jsonResult.get("hourly");
            for (int i = 0; i < jsonArray.size(); i++) {
                JSONObject jsonObject = (JSONObject) jsonArray.get(i);
                String time = (String) jsonObject.get("time");
                time = time.replaceAll("\\+08:00", "").replaceAll("T", " ");
                BigDecimal temperature = new BigDecimal((String) jsonObject.get("temperature"));
                //新建需要保存的对象
                BusTemperature b = new BusTemperature();
                b.setCode(code);
                b.setTm(sdf.parse(time));
                b.setTemperature(temperature);
                //存到list
                resultList.add(b);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            try {
                input.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return resultList;
    }

}
