package org.rcisoft.business.mainte.distributed.dao;

import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.mainte.distributed.entity.BuildingAge;
import org.rcisoft.business.mainte.distributed.entity.DeviceAge;
import org.rcisoft.business.mainte.distributed.entity.EnergyCompared;
import org.rcisoft.business.mainte.distributed.entity.EnergyDistributed;
import org.rcisoft.business.mainte.distributed.vo.Params;
import org.springframework.stereotype.Repository;

import java.util.List;

/**
 * Created by JiChao on 2018/4/23.
 */
@Repository
public interface MainteDistributedRepository {

    @Select("<script>select b.PRO_NM,b.BLD_COORDINATE,b.BLD_AREA, " +
            "sum(if(e.WATER_MONEY is null, 0, e.WATER_MONEY)) WATER_MONEY, " +
            "sum(if(e.ELEC_MONEY is null, 0, e.ELEC_MONEY)) ELEC_MONEY, " +
            "sum(if(e.GAS_MONEY is null, 0, e.GAS_MONEY)) GAS_MONEY " +
            "from energy_count_m e,bus_project b " +
            "where e.PRO_ID=b.PRO_ID and e.`YEAR`=#{year} and e.MON=${month} " +
            "group by e.MON,b.PRO_NM</script>")
    @ResultMap(value = "EnergyDistributed")
    List<EnergyDistributed> queryEnergyDistributed(Params params);

    @Select("<script>select a.BLD_TP_NM,a.MON,convert(avg(a.MONEY),decimal(10,2)) MONEY from (" +
            "select e.PRO_ID,t.BLD_TP_NM,e.MON," +
            "sum((if(e.WATER_MONEY is null, 0, e.WATER_MONEY)+if(e.ELEC_MONEY is null, 0, e.ELEC_MONEY)+if(e.GAS_MONEY is null, 0, e.GAS_MONEY))/b.BLD_AREA) MONEY " +
            "from energy_count_m e,bus_project b,bus_build_tp t " +
            "where e.PRO_ID=b.PRO_ID and b.BLD_TP_ID=t.BLD_TP_ID and e.`YEAR`=#{year} and b.BLD_ZONE_ID=#{bldZoneId} " +
            "group by e.PRO_ID,t.BLD_TP_NM,e.MON order by t.BLD_TP_NM,e.MON" +
            ") a group by a.MON,a.BLD_TP_NM order by a.BLD_TP_NM,a.MON</script>")
    @ResultMap(value = "EnergyCompared")
    List<EnergyCompared> queryEnergyCompared(Params params);

    @Select("<script>select b.PRO_NM,t.BLD_TP_NM,date_format(b.BLD_YEAR, '%Y') `YEAR`," +
            "convert(sum((if(e.WATER_MONEY is null, 0, e.WATER_MONEY)+if(e.ELEC_MONEY is null, 0, e.ELEC_MONEY)+if(e.GAS_MONEY is null, 0, e.GAS_MONEY))/b.BLD_AREA),decimal(10,2)) MONEY " +
            "from energy_count_m e,bus_project b,bus_build_tp t " +
            "where e.PRO_ID=b.PRO_ID and b.BLD_TP_ID=t.BLD_TP_ID  " +
            "group by e.PRO_ID,b.PRO_NM,t.BLD_TP_NM,date_format(b.BLD_YEAR, '%Y') " +
            "order by t.BLD_TP_NM</script>")
    @ResultMap(value = "BuildingAge")
    List<BuildingAge> queryBuildingAge();

    @Select("<script>select b.PRO_NM,t.BLD_TP_NM,date_format(b.DEVICE_YEAR, '%Y') `YEAR`," +
            "convert(sum((if(e.WATER_MONEY is null, 0, e.WATER_MONEY)+if(e.ELEC_MONEY is null, 0, e.ELEC_MONEY)+if(e.GAS_MONEY is null, 0, e.GAS_MONEY))/b.BLD_AREA),decimal(10,2)) MONEY " +
            "from energy_count_m e,bus_project b,bus_build_tp t " +
            "where e.PRO_ID=b.PRO_ID and b.BLD_TP_ID=t.BLD_TP_ID  " +
            "group by e.PRO_ID,b.PRO_NM,t.BLD_TP_NM,date_format(b.DEVICE_YEAR, '%Y') " +
            "order by t.BLD_TP_NM</script>")
    @ResultMap(value = "DeviceAge")
    List<DeviceAge> queryDeviceAge();

}
