package org.rcisoft.business.mainte.energyplan.controller;


/*固定导入*/
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyPlan;
import org.rcisoft.business.mainte.energyplan.service.BusEnergyPlanService;
import org.rcisoft.business.mainte.energyplan.vo.EnergyPlanList;
import org.rcisoft.business.mainte.energyplan.vo.EnergyPlanVo;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;


/**
* Created by  on 2018-4-17 17:47:37.
*/
@RestController
@RequestMapping("busenergyplan")
public class BusEnergyPlanController extends PaginationController<BusEnergyPlan> {

    @Autowired
    private BusEnergyPlanService busEnergyPlanServiceImpl;


    @ApiOperation(value="添加能耗计划", notes="添加能耗计划")
    @PostMapping(value = "/add")
    public Result add(@RequestBody EnergyPlanList energyPlanlist) throws Exception{
        List<EnergyPlanVo> epList = energyPlanlist.getEnergyPlanVoList();
        for(EnergyPlanVo energyPlanVo : epList){
            busEnergyPlanServiceImpl.save(energyPlanVo);
        }
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
                null);
    }

    @ApiOperation(value="删除设备某一天的能耗计划", notes="删除设备某一天的能耗计划")
    @DeleteMapping("/delete")
    public Result delete(@RequestParam String devNum,@RequestParam String day) {
        PersistModel data = busEnergyPlanServiceImpl.remove(devNum,day);
        return Result.builder(data,
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        null);
    }

    @ApiOperation(value="根据项目查询某一天计划总能耗", notes="根据项目查询某一天计划总能耗")
    @GetMapping(value = "/queryEnergyPlanDay")
    public Result queryEnergyPlanDay(@RequestParam String pro_id,@RequestParam String day){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryEnergyPlanDay(pro_id,day));
    }
    @ApiOperation(value="查询能耗计划", notes="查询能耗计划")
    @GetMapping(value = "/queryEnergyPlanList")
    public Result queryEnergyPlanList(@RequestParam String pro_id,@RequestParam String day)throws Exception{
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryEnergyPlanList(pro_id,day));
    }
    @ApiOperation(value="查询设备类型", notes="查询设备类型")
    @GetMapping(value = "/queryDeviceTp")
    public Result queryDeviceTp(){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryDeviceTp());
    }

    @ApiOperation(value="根据设备类型查询设备", notes="根据设备类型查询设备")
    @GetMapping(value = "/queryDeviceByTp")
    public Result queryDeviceByTp(@RequestParam String dev_tp_id){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryDeviceByTp(dev_tp_id));
    }

    @ApiOperation(value="根据项目查询某一天实际总能耗", notes="根据项目查询某一天实际总能耗")
    @GetMapping(value = "/queryEnergyRealDay")
    public Result queryEnergyRealDay(@RequestParam String pro_id,@RequestParam String day){
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryBusEnergyRealCount(pro_id,day));
    }


    @ApiOperation(value="用能计划执行", notes="用能计划执行")
    @GetMapping(value = "/queryEnergyPlanExecute")
    public Result queryEnergyPlanExecute(@RequestParam String pro_id,@RequestParam String day)throws Exception{
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryEnergyPlanExecute(pro_id,day));
    }

    @ApiOperation(value="设备用能功率对比", notes="设备用能功率对比")
    @GetMapping(value = "/queryEnergyPowerCompare")
    public Result queryEnergyPowerCompare(@RequestParam String dev_num,@RequestParam String day)throws Exception{
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.queryEnergyPowerCompare(dev_num,day));
    }

    @ApiOperation(value="设备参数库操作（录入开关）", notes="设备参数库操作（录入开关）")
    @GetMapping(value = "/paramLibraryOperate")
    public Result paramLibraryOperate(@RequestParam String dev_num,@RequestParam String pl_switch){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("pl_switch",pl_switch);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busEnergyPlanServiceImpl.paramLibraryOperate(map));
    }
}
