package org.rcisoft.business.mainte.energyplan.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import org.rcisoft.business.mainte.energyplan.dao.BusEnergyPlanRepository;
import org.rcisoft.business.mainte.energyplan.entity.BusEnergyPlan;
import org.rcisoft.business.mainte.energyplan.service.BusEnergyPlanService;
import org.rcisoft.business.mainte.energyplan.vo.EnergyPlanVo;
import org.rcisoft.business.mainte.paramlibrary.dao.ParamLibraryRepository;
import org.rcisoft.core.util.UploadUtil;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;



import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

import lombok.extern.slf4j.Slf4j;
import tk.mybatis.mapper.entity.Example;

/**
 * Created by  on 2018-4-17 17:47:37.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusEnergyPlanServiceImpl implements BusEnergyPlanService {

    @Autowired
    private BusEnergyPlanRepository busEnergyPlanRepository;
    @Autowired
    private ParamLibraryRepository paramLibraryRepository;
    @Value("${serverimgurl}")
    private String serverImgUrl;


    /**
    * 保存 busEnergyPlan
    * @param
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(EnergyPlanVo energyPlanVo)throws Exception{
        BusEnergyPlan busEnergyPlan = new BusEnergyPlan();
        //时间拆分，10分钟一条记录
        ArrayList timeArr = new ArrayList();
        Calendar c = Calendar.getInstance();
        SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date sTime = simple.parse(energyPlanVo.getStartTm());
        Date eTime = simple.parse(energyPlanVo.getEndTm());
        while(!sTime.equals(eTime)){
            timeArr.add(sTime);
            c.setTime(sTime);
            c.add(Calendar.MINUTE,10);
            sTime = c.getTime();
        }
        String[] paramS = energyPlanVo.getParamStr().split(",");
        String[] paramValS = energyPlanVo.getParamValStr().split(",");
        for(int i=0;i<paramS.length;i++){
            Map<String,Object> tmap = new HashMap<String,Object>();
            tmap.put("dev_num",energyPlanVo.getDevNum());
            tmap.put("param",paramS[i]);
            List<Map<String,Object>> tlist = busEnergyPlanRepository.queryEnergyPlanCompareFlag(tmap);
            if(tlist!=null && tlist.size()>0 && tlist.get(0).get("compare_flag") != null && tlist.get(0).get("compare_flag").toString().equalsIgnoreCase("1")){
                busEnergyPlan.setParam(paramS[i]);
                busEnergyPlan.setParamVal(Float.valueOf(paramValS[i]));
            }
        }
        ArrayList<BusEnergyPlan> list = new ArrayList<BusEnergyPlan>();
        int len = paramValS.length;
        Float p1_v = null;
        Float p2_v = null;
        Float p3_v = null;
        if(len==3){
            p1_v = Float.valueOf(paramValS[0]);
            p2_v = Float.valueOf(paramValS[1]);
            p3_v = Float.valueOf(paramValS[2]);
        }else if(len==2){
            p1_v = Float.valueOf(paramValS[0]);
            p2_v = Float.valueOf(paramValS[1]);
        }else if(len==1){
            p1_v = Float.valueOf(paramValS[0]);
        }
        Map<String,Object> m = new HashMap<String,Object>();
        m.put("p1_v",p1_v);
        m.put("p2_v",p2_v);
        m.put("p3_v",p3_v);
        m.put("dev_num",energyPlanVo.getDevNum());
        m.put("year",simple.format(new Date()).substring(0,4));
        List<Map<String,Object>> paramlibrary = paramLibraryRepository.queryParamLibrary(m);
        for(Object tm : timeArr){
            busEnergyPlan.setDevNum(energyPlanVo.getDevNum());
            busEnergyPlan.setTm((Date)tm);
            if(paramlibrary!=null && paramlibrary.size()>0){
                busEnergyPlan.setPower(Float.valueOf(paramlibrary.get(0).get("power").toString()));
                busEnergyPlan.setGasSpeed(Float.valueOf(paramlibrary.get(0).get("gas_speed").toString()));
            }
            Map<String,Object> tmp = new HashMap<String,Object>();
            tmp.put("dev_num",energyPlanVo.getDevNum());
            tmp.put("tm",tm);
            List<Map<String,Object>> existPlan = busEnergyPlanRepository.queryExistEnergyPlan(tmp);
            if(existPlan==null || existPlan.size()<=0){
                busEnergyPlanRepository.insertSelective(busEnergyPlan);
            }
        }
        return new PersistModel(1);
    }

    /**
    * 逻辑删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(String devNum,String day){
        int line = 1;
        Map<String,Object> map = new HashMap<>();
        map.put("dev_num",devNum);
        map.put("day",day);
        try {
            busEnergyPlanRepository.deleteEnergyPlanDay(map);
        }catch(Exception e){
            line = 0;
        }
        return new PersistModel(line);
    }



    @Override
    public List<Map<String, Object>> queryEnergyPlanDay(String proid, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        return busEnergyPlanRepository.queryEnergyPlanDay(map);
    }

    @Override
    public List<Map<String, Object>> queryEnergyPlanList(String proid, String day)throws Exception {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        List<Map<String, Object>> list = busEnergyPlanRepository.queryEnergyPlanList(map);
        ArrayList timeArr = new ArrayList();
        Calendar c = Calendar.getInstance();
        SimpleDateFormat simple = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Date sTime = simple.parse(day + " 00:00:00");
        Date eTime = simple.parse(day + " 23:50:00");
        while(!sTime.equals(eTime)){
            timeArr.add(simple.format(sTime));
            c.setTime(sTime);
            c.add(Calendar.MINUTE,10);
            sTime = c.getTime();
        }
            timeArr.add(day + " 23:50:00");
        for(Map<String, Object> tmap : list){
            Map<String,Object> tmMap = new HashMap<String,Object>();
            for(Object tm : timeArr){
                tmMap.put(tm.toString(),null);
            }
            String[] ts = tmap.get("tm_str").toString().split(",");
            for(String s : ts){
                tmMap.put(s,"1");
            }
            tmap.put("planmap",tmMap);

        }
        return list;
    }

    @Override
    public List<Map<String, Object>> queryDeviceTp() {
        List<Map<String, Object>> list = busEnergyPlanRepository.queryDeviceTp();
        for(Map<String, Object> m : list){
            if(m.get("DEV_TP_IMG")!=null && !m.get("DEV_TP_IMG").toString().equals("") && !m.get("DEV_TP_IMG").toString().equals("null")){
                String type = "";
                String img = UploadUtil.GetImageStr(m.get("DEV_TP_IMG").toString());
                m.put("IMG", img);
            }else{
                m.put("IMG","");
            }
        }
        return list;
    }

    @Override
    public List<Map<String, Object>> queryDeviceByTp(String dev_tp_id) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_tp_id",dev_tp_id);
        return busEnergyPlanRepository.queryDeviceByTp(map);
    }

    @Override
    public List<Map<String, Object>> queryBusEnergyRealCount(String proid, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        return busEnergyPlanRepository.queryBusEnergyRealCount(map);
    }

    @Override
    public List<Map<String, Object>> queryEnergyPlanExecute(String proid, String day)throws Exception {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",proid);
        map.put("day",day);
        Map<String,Object> sensorMap = new HashMap<>();
        List<Map<String,Object>> sensorList = busEnergyPlanRepository.querySensor(map);
        for(Map<String,Object> m : sensorList){
            sensorMap.put(m.get("tm").toString(),m.get("sensor_json"));
        }
        List<Map<String,Object>> executeList = busEnergyPlanRepository.queryBusEnergyPlanExecute(map);
        Map<String,Object> executeMap = new HashMap<String,Object>();
        for(Map<String,Object> tmap : executeList){
            //设备状态 0：状态不一致 1：参数不一致  2：完全一致
            //状态由耗电量决定
            String flag = "2";
            //10分钟内设备耗电量为空或小于0.5度，设备默认为关闭状态
            if(tmap.get("elec")==null || tmap.get("elec").toString().equalsIgnoreCase("") || Float.valueOf(tmap.get("elec").toString()) <= 0.5){
                flag = "0";
            }else{
                String param = tmap.get("param").toString();
                String param_val = tmap.get("param_val")==null?"":tmap.get("param_val").toString();
                String compare_val = "";
                if(param.contains("sensor")){//要对比的参数取自于传感器
                    if(sensorMap.get(tmap.get("tm").toString())==null){
                        compare_val = "";
                    }else{
                        String sensorJson = sensorMap.get(tmap.get("tm")).toString();
                        JSONObject sensorBody = JSONArray.parseObject(sensorJson);
                        /**
                         * 为了区分设备参数与传感器参数，在为设备配置模板的时候会为来源于传感器的
                         * 参数前面自动拼接sensor_ ，当然，从传感器中取数据的时候需要去掉拼接的东西
                         */
                        String t_param = param.substring(param.indexOf("_")+1,param.length());
                        BigDecimal p_v = new BigDecimal(sensorBody.get(t_param)==null?"0":sensorBody.get(t_param).toString());
                        compare_val = p_v.setScale(1).toString();
                    }

                }else{//要对比的参数取自于设备本身
                    String[] numArr = {"4","5","6","7","8","9","10","11","12","13","14","15","16","17","18","19","20"};
                    for(String num : numArr){
                        if(param.equalsIgnoreCase(tmap.get("p"+num)==null?"":tmap.get("p"+num).toString())){
                            compare_val = tmap.get("p"+num+"_v")==null?"":tmap.get("p"+num+"_v").toString();
                        }
                    }
                }
                if(!param_val.equalsIgnoreCase(compare_val)){
                    flag = "1";
                }
                if(param_val.equalsIgnoreCase("") && compare_val.equalsIgnoreCase("")){
                    flag = "1";
                }
            }
            executeMap.put(tmap.get("dev_num").toString()+","+tmap.get("tm").toString(),flag);
        }
        List<Map<String, Object>> planList = queryEnergyPlanList(proid,day);
        for(Map<String, Object> plan : planList){
            String dev_num = plan.get("dev_num").toString();
            Map<String, Object> planMap = (Map<String, Object>)plan.get("planmap");
            for (Map.Entry<String, Object> entry : planMap.entrySet()) {
                planMap.put(entry.getKey(),executeMap.get(dev_num+","+entry.getKey()));
            }
        }
        return planList;
    }

    @Override
    public List<Map<String, Object>> queryEnergyPowerCompare(String dev_num, String day) {
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("day",day);
        return busEnergyPlanRepository.queryEnergyPowerCompare(map);
    }


    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public int paramLibraryOperate(Map<String, Object> map) {
        String pl_switch = map.get("pl_switch").toString();
        if(pl_switch.equalsIgnoreCase("1")){//参数库录入：开
            SimpleDateFormat simple = new SimpleDateFormat("yyyy");
            map.put("year",simple.format(new Date()));
            busEnergyPlanRepository.deleteParamLibrary(map);
        }
        busEnergyPlanRepository.updateDevicePswitch(map);
        return 1;
    }


}
