package org.rcisoft.business.mainte.paramlibrary.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.mainte.paramlibrary.service.ParamLibraryService;
import org.rcisoft.business.mainte.paramlibrary.vo.BusParamLibraryList;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("paramlibrary")
public class ParamLibraryController {
    @Autowired
    private ParamLibraryService paramLibraryService;

    @ApiOperation(value="根据设备编号获取参数库记录参数", notes="根据设备编号获取参数库记录参数")
    @GetMapping(value = "/queryLibraryParamByDev")
    public Result queryLibraryParamByDev(@RequestParam String dev_num,@RequestParam String pro_id){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("pro_id",pro_id);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                paramLibraryService.queryLibraryParamByDev(map));
    }

    @ApiOperation(value="查询参数1", notes="查询参数1")
    @GetMapping(value = "/queryPlanParamVal_1")
    public Result queryPlanParamVal_1(@RequestParam String dev_num){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                paramLibraryService.queryPlanParamVal_1(map));
    }

    @ApiOperation(value="查询参数2", notes="查询参数2")
    @GetMapping(value = "/queryPlanParamVal_2")
    public Result queryPlanParamVal_2(@RequestParam String dev_num,@RequestParam String p1_v){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("p1_v",p1_v);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                paramLibraryService.queryPlanParamVal_2(map));
    }

    @ApiOperation(value="查询参数3", notes="查询参数3")
    @GetMapping(value = "/queryPlanParamVal_3")
    public Result queryPlanParamVal_3(@RequestParam String dev_num,@RequestParam String p1_v,@RequestParam String p2_v){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        map.put("p1_v",p1_v);
        map.put("p2_v",p2_v);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                paramLibraryService.queryPlanParamVal_3(map));
    }

    @ApiOperation(value="设备参数库", notes="设备参数库")
    @GetMapping(value = "/queryParamLibrary")
    public Result queryParamLibrary(@RequestParam String dev_num){
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("dev_num",dev_num);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                paramLibraryService.queryParamLibrary(map));
    }

    @ApiOperation(value="添加", notes="添加")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/add")
    public Result add(@RequestBody BusParamLibraryList busParamLibraryList) {
        PersistModel data = paramLibraryService.save(busParamLibraryList);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busParamLibraryList);
    }

    @ApiOperation(value="清空设备参数库", notes="清空设备参数库")
    //@ApiImplicitParams({@ApiImplicitParam(name = "businessId", value = "businessId", required = false, dataType = "varchar")})
    @PostMapping(value = "/delete")
    public Result delete(@RequestParam String devNum) {
        PersistModel data = paramLibraryService.delete(devNum);
        return Result.builder(data,
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                devNum);
    }

    @ApiOperation(value="下载参数库模板", notes="下载参数库模板")
    @ApiImplicitParams({@ApiImplicitParam(name = "devNum", value = "设备编号", required = true, dataType = "字符串")})
    @RequestMapping("/downloadExcel/{devNum:\\w+}")
    public void downloadExcel(HttpServletResponse response, @PathVariable String devNum) {
        paramLibraryService.downloadExcel(response, devNum);
    }

    @ApiOperation(value="上传参数库模板", notes="上传参数库模板")
    @ApiImplicitParams({@ApiImplicitParam(name = "devNum", value = "设备编号", required = true, dataType = "字符串")})
    @RequestMapping("/uploadExcel")
    public Result uploadExcel(@RequestParam MultipartFile file, @RequestParam String devNum) {
        Integer result = paramLibraryService.uploadExcel(file, devNum);
        return Result.builder(new PersistModel(result), MessageConstant.MESSAGE_ALERT_SUCCESS, MessageConstant.MESSAGE_ALERT_ERROR, result);
    }
}
