package org.rcisoft.business.mainte.paramlibrary.dao;

import org.apache.ibatis.annotations.Insert;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;
import org.rcisoft.business.mainte.paramlibrary.entity.BusParamLibrary;
import org.rcisoft.core.base.BaseMapper;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

@Repository
public interface ParamLibraryRepository extends BaseMapper<BusParamLibrary>{
    /**
     * 根据设备编号获取参数库记录参数
     * @return
     */
    @Select("<script>SELECT\n" +
            "\ta.param,\n" +
            "\tc.param_nm\n" +
            "FROM\n" +
            "\tbus_library_param a,\n" +
            "\tbus_device_param b,\n" +
            "\tbus_devicetp_param c\n" +
            "WHERE\n" +
            "\ta.DEV_NUM = b.DEV_NUM\n" +
            "AND a.PARAM = b.PARAM\n" +
            "AND b.PARAM_CODE = c.PARAM_CODE\n" +
            "AND a.DEV_NUM = #{dev_num} and c.PRO_ID = #{pro_id} "
            + "</script>")
    List<Map<String,Object>> queryLibraryParamByDev(Map<String,Object> map);

    /**
     * 根据设备编码获取第一个参数集合
     * @return
     */
    @Select("<script>select distinct p1_v from bus_param_library where dev_num = #{dev_num} and year = #{year} "
            + "</script>")
    List<Map<String,Object>> queryPlanParamVal_1(Map<String,Object> map);
    /**
     * 根据设备编码及第一个参数获取第二个参数集合
     * @return
     */
    @Select("<script>select distinct p1_v,p2_v,p3_v,power,gas_speed from bus_param_library where dev_num = #{dev_num}" +
            " and p1_v = #{p1_v}  and year = #{year} "
            + "</script>")
    List<Map<String,Object>> queryPlanParamVal_2(Map<String,Object> map);
    /**
     * 根据设备编码及前两个参数获取第三个参数集合
     * @return
     */
    @Select("<script>select distinct p1_v,p2_v,p3_v,power,gas_speed from bus_param_library where dev_num = #{dev_num}" +
            " and p1_v = #{p1_v} and p2_v = #{p2_v}  and year = #{year} "
            + "</script>")
    List<Map<String,Object>> queryPlanParamVal_3(Map<String,Object> map);

    /**
     * 查询参数库
     * @param map
     * @return
     */
    @Select("<script>SELECT \n" +
            "\tDISTINCT\n" +
            "\ta.p1,a.p1_v,a.p2,a.p2_v,a.p3,a.p3_v,a.power,a.gas_speed,\n" +
            "\tb1.PARAM_NM p1_nm,b2.PARAM_NM p2_nm,b3.PARAM_NM p3_nm,\n" +
            "\ta.year\n" +
            "FROM\n" +
            "\tbus_param_library a\n" +
            "left join bus_device_param c1 on a.p1 = c1.param and c1.dev_num = a.dev_num " +
            "left join bus_device_param c2 on a.p2 = c2.param and c1.dev_num = a.dev_num " +
            "left join bus_device_param c3 on a.p3 = c3.param and c1.dev_num = a.dev_num " +
            "left join bus_devicetp_param b1 on c1.param_code = b1.param_code\n" +
            "left join bus_devicetp_param b2 on c2.param_code = b2.param_code\n" +
            "left join bus_devicetp_param b3 on c3.param_code = b3.param_code " +
            "where a.dev_num = #{dev_num} " +
            " <if test = 'p1_v != null '> and p1_v = #{p1_v} </if> " +
            " <if test = 'p2_v != null '> and p2_v = #{p2_v} </if> " +
            " <if test = 'p3_v != null '> and p3_v = #{p3_v} </if> " +
            " <if test = 'year != null '> and year = #{year} </if> "
            + "</script>")
    List<Map<String,Object>> queryParamLibrary(Map<String,Object> map);

    @Insert("<script><foreach collection=\"resultList\" item=\"item\" separator=\";\">" +
            "insert into bus_param_library(DEV_NUM,P1,P1_V,P2,P2_V,P3,P3_V,POWER,GAS_SPEED,YEAR) " +
            "values(#{item.devNum},#{item.p1},#{item.p1V},#{item.p2},#{item.p2V},#{item.p3},#{item.p3V},#{item.power},#{item.gasSpeed},#{item.year})" +
            "</foreach></script>")
    @ResultType(value = Integer.class)
    int saveBusParamLibraryList(@Param(value = "resultList") List<BusParamLibrary> resultList);
}
