package org.rcisoft.business.mainte.paramlibrary.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFRow;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellType;
import org.rcisoft.business.mainte.paramlibrary.dao.ParamLibraryRepository;
import org.rcisoft.business.mainte.paramlibrary.entity.BusParamLibrary;
import org.rcisoft.business.mainte.paramlibrary.service.ParamLibraryService;
import org.rcisoft.business.mainte.paramlibrary.vo.BusParamLibraryList;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import tk.mybatis.mapper.entity.Example;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class ParamLibraryServiceImpl implements ParamLibraryService {
    @Autowired
    private ParamLibraryRepository paramLibraryRepository;
    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;

    @Override
    public List<Map<String, Object>> queryLibraryParamByDev(Map<String, Object> map) {
        return paramLibraryRepository.queryLibraryParamByDev(map);
    }

    @Override
    public List<Map<String, Object>> queryPlanParamVal_1(Map<String, Object> map) {
        SimpleDateFormat simple = new SimpleDateFormat("yyyy");
        map.put("year",simple.format(new Date()));
        return paramLibraryRepository.queryPlanParamVal_1(map);
    }

    @Override
    public List<Map<String, Object>> queryPlanParamVal_2(Map<String, Object> map) {
        SimpleDateFormat simple = new SimpleDateFormat("yyyy");
        map.put("year",simple.format(new Date()));
        return paramLibraryRepository.queryPlanParamVal_2(map);
    }

    @Override
    public List<Map<String, Object>> queryPlanParamVal_3(Map<String, Object> map) {
        SimpleDateFormat simple = new SimpleDateFormat("yyyy");
        map.put("year",simple.format(new Date()));
        return paramLibraryRepository.queryPlanParamVal_3(map);
    }

    @Override
    public List<Map<String, Object>> queryParamLibrary(Map<String, Object> map) {
        return paramLibraryRepository.queryParamLibrary(map);
    }

    @Override
    public PersistModel save(BusParamLibraryList busPramLibraryList) {
        for(BusParamLibrary bp : busPramLibraryList.getBusParamLibraryList()){
            Map<String,Object> map = new HashMap<>();
            map.put("dev_num",bp.getDevNum());
            map.put("p1_v",bp.getP1V());
            map.put("p2_v",bp.getP2V());
            map.put("p3_v",bp.getP3V());
            map.put("year",bp.getYear());
            if(paramLibraryRepository.queryParamLibrary(map)==null || paramLibraryRepository.queryParamLibrary(map).size()<1) {
                paramLibraryRepository.insertSelective(bp);
            }
        }
        return new PersistModel(1);
    }

    @Override
    public PersistModel delete(String devNum) {
        Example example = new Example(BusParamLibrary.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devNum",devNum);
        int line = paramLibraryRepository.deleteByExample(example);
        return new PersistModel(line);
    }

    @Override
    public void downloadExcel(HttpServletResponse response, String devNum) {
        try(OutputStream outputStream = response.getOutputStream()) {
            //设置下载头
            response.setHeader("Content-disposition", "attachment;filename=" + devNum + ".xls");
            outputStream.write(this.createExcel(devNum));
            outputStream.flush();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Transactional
    @Override
    public Integer uploadExcel(MultipartFile file, String devNum) {
        HSSFWorkbook workbook = null;
        List<BusParamLibrary> resultList = new ArrayList<>();
        int result = 0;
        try(InputStream is = file.getInputStream()) {
            workbook = new HSSFWorkbook(is);
            //读取sheet
            HSSFSheet sheet = workbook.getSheetAt(0);
            //从第一行开始读取
            HSSFRow first_row = sheet.getRow(0);
            //第一行得到devNum，年份
            String excel_devNum = "";
            String year = "";
            if (first_row != null) {
                excel_devNum = this.getValue(first_row.getCell(1));
                year = this.getValue(first_row.getCell(4));
                //如果不等于页面传来的值，直接返回
                if (!StringUtils.equals(excel_devNum, devNum)) return result;
            }
            //取得三个参数的code
            String code1 = this.getCode(sheet, 3),
                    code2 = this.getCode(sheet, 4),
                    code3 = this.getCode(sheet, 5);
            //如果code1、2、3都没有值，表示没有添加参数模板
            if (code1 == null && code2 == null && code3 == null) {
                
            } else {
                //读取需要保存的数据
                for (int i = 8; i < sheet.getLastRowNum() + 1; i++) {
                    BusParamLibrary b = new BusParamLibrary();
                    HSSFRow row = sheet.getRow(i);
                    if (row != null) {
                        String power = this.getValue(row.getCell(1));//功率
                        String speed = this.getValue(row.getCell(2));//用气速率
                        String code1Value = this.getValue(row.getCell(3));//参数1
                        String code2Value = this.getValue(row.getCell(4));//参数2
                        String code3Value = this.getValue(row.getCell(5));//参数3
                        //如果都为null，表明已经没有数据，跳出循环
                        if (power == null && speed == null && code1Value == null && code2Value == null && code3Value == null) break;
                        //赋值
                        b.setDevNum(devNum);
                        b.setPower(power == null ? null : new BigDecimal(power));
                        b.setGasSpeed(speed == null ? null : new BigDecimal(speed));
                        b.setYear(year);
                        b.setP1(code1);
                        b.setP1V(code1Value == null ? null : new BigDecimal(code1Value));
                        b.setP2(code2);
                        b.setP2V(code2Value == null ? null : new BigDecimal(code2Value));
                        b.setP3(code3);
                        b.setP3V(code3Value == null ? null : new BigDecimal(code3Value));
                        resultList.add(b);
                    }
                }
                //先删除数据库的数据，再保存
                if (resultList.size() > 0) {
                    BusParamLibrary b = new BusParamLibrary();
                    b.setDevNum(devNum);
                    b.setYear(year);
                    //先删除
                    paramLibraryRepository.delete(b);
                    //添加
                    result = paramLibraryRepository.saveBusParamLibraryList(resultList);
                }
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    /**
     * 创建下载的excel流
     * @param devNum
     * @return
     */
    private byte[] createExcel(String devNum) throws IOException {
        //定义行号
        int rowNum = 0;
        HSSFWorkbook workbook = new HSSFWorkbook();
        HSSFSheet sheet = workbook.createSheet("sheet1");
        //第一行：设备编号 + 年份
        HSSFRow row_1 = sheet.createRow(rowNum++);
        row_1.createCell(0, CellType.STRING).setCellValue("设备编号");
        row_1.createCell(1, CellType.STRING).setCellValue(devNum);
        row_1.createCell(3, CellType.STRING).setCellValue("年份");
        row_1.createCell(4, CellType.STRING).setCellValue(Calendar.getInstance().get(Calendar.YEAR));
        //空一行
        rowNum++;
        // 第三行：参数、参数名称、参数编码
        String[] title_3 = new String[]{"参数", "名称", "编码"};
        HSSFRow row_3 = sheet.createRow(rowNum++);
        for (int i = 0; i < title_3.length; i++) {
            row_3.createCell(i, CellType.STRING).setCellValue(title_3[i]);
        }
        //循环参数列表
        List<Map<String, String>> paramsList = busLibraryParamRepository.selectParamsByDevNum(devNum);
        int size = paramsList.size();
        for (int i = 0; i < size; i++) {
            Map<String, String> param = paramsList.get(i);
            HSSFRow row = sheet.createRow(rowNum++);
            row.createCell(0, CellType.STRING).setCellValue(i + 1);//编号
            row.createCell(1, CellType.STRING).setCellValue(param.get("paramNm"));//参数名称
            row.createCell(2, CellType.STRING).setCellValue(param.get("param"));//参数编码
        }
        //空一行
        rowNum = rowNum + 1 + 3 - size;
        //添加参数值表头列
        HSSFRow row_next = sheet.createRow(rowNum);
        row_next.createCell(0, CellType.STRING).setCellValue("参数值列表");
        row_next.createCell(1, CellType.STRING).setCellValue("功率（kw）");
        row_next.createCell(2, CellType.STRING).setCellValue("用气速率（m3/h）");
        for (int i = 0; i < size; i++) {
            row_next.createCell(3 + i, CellType.STRING).setCellValue("参数" + (i + 1) + "值");
        }
        try(ByteArrayOutputStream os = new ByteArrayOutputStream()) {
            workbook.write(os);
            return os.toByteArray();
        } finally {
            workbook.close();
        }
    }

    /**
     * 读取cell的内容
     * @param cell
     * @return
     */
    @SuppressWarnings("deprecation")
    private String getValue(HSSFCell cell) {
        if (cell == null) return null;
        switch (cell.getCellType()) {
            case Cell.CELL_TYPE_BOOLEAN:
                return cell.getBooleanCellValue() ? "TRUE" : "FALSE";
            case Cell.CELL_TYPE_FORMULA:
                return cell.getCellFormula();
            case Cell.CELL_TYPE_NUMERIC:
                cell.setCellType(Cell.CELL_TYPE_STRING);
                return cell.getStringCellValue();
            case Cell.CELL_TYPE_STRING:
                return cell.getStringCellValue();
            default:
                return null;
        }
    }

    /**
     * 读取code
     * @param sheet
     * @param row
     * @return
     */
    private String getCode(HSSFSheet sheet, int row) {
        HSSFRow sheetRow = sheet.getRow(row);
        if (sheetRow == null) return null;
        else return this.getValue(sheetRow.getCell(2));
    }
}
