package org.rcisoft.business.manage.controller;


/*固定导入*/
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.serializer.SerializerFeature;
import io.jsonwebtoken.Claims;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.service.BusProjectService;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.core.constant.MessageConstant;
import org.rcisoft.core.controller.PaginationController;
import org.rcisoft.core.model.GridModel;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.result.Result;
import org.rcisoft.core.util.JwtUtil;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.propertyeditors.CustomDateEditor;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.web.bind.WebDataBinder;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.context.request.WebRequest;

import javax.validation.Valid;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
* Created by  on 2018-5-2 16:09:30.
*/
@RestController
@RequestMapping("manage/busproject")
public class BusProjectController extends PaginationController<BusProject> {

    @Autowired
    private BusProjectService busProjectServiceImpl;


    @ApiOperation(value="添加项目", notes="添加项目")
    @PostMapping(value = "/add")
    public Result add(@Valid BusProject busProject) {
        PersistModel data = busProjectServiceImpl.save(busProject);
        return Result.builder(data);
    }


    @ApiOperation(value="修改项目信息", notes="修改项目信息")
    @PutMapping("/update")
    public Result update(@Valid BusProject busProject) {
        PersistModel data = busProjectServiceImpl.merge(busProject);
        return Result.builder(data);
    }

    @ApiOperation(value="删除项目", notes="删除项目")
    @DeleteMapping("/delete")
    public Result delete(@RequestParam String proId) {
        BusProject busProject = new BusProject();
        busProject.setProId(proId)  ;
        PersistModel data = busProjectServiceImpl.delete(busProject);
        return Result.builder(data);
    }

    @ApiOperation(value="根据条件查询单个项目信息", notes="根据条件查询单个项目信息")
    @GetMapping("/detail")
    public Result detail(@Valid BusProject busProject) {
        return Result.builder(new PersistModel(1),
            MessageConstant.MESSAGE_ALERT_SUCCESS,
            MessageConstant.MESSAGE_ALERT_ERROR,
        busProjectServiceImpl.findByParam(busProject));
    }

    @ApiOperation(value="查看  集合", notes="查看  集合")
    @GetMapping(value = "/queryBusProjectByPagination")
    public GridModel listByPagination(BusProject busProject) {
        busProjectServiceImpl.findAllByPagination(getPaginationUtility(), busProject);
        return getGridModelResponse();
    }

    @ApiOperation(value="查询用户权限项目", notes="查询用户权限项目")
    @GetMapping(value = "/queryBusProjectByUser")
    public Result queryBusProjectByUser() {
        String userId = UserUtil.getUserInfoProp(getToken(),UserUtil.USER_ID);
        Map<String,Object> map = new HashMap<>();
        map.put("user_id",userId);
        return Result.builder(new PersistModel(1),
                MessageConstant.MESSAGE_ALERT_SUCCESS,
                MessageConstant.MESSAGE_ALERT_ERROR,
                busProjectServiceImpl.queryBusProjectByUser(map));
    }

    @InitBinder
    public void initBinder(WebDataBinder webDataBinder, WebRequest webRequest) {
        DateFormat dateFormat=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        webDataBinder.registerCustomEditor(Date.class, new CustomDateEditor(dateFormat, true));
    }
}
