package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.manage.dao.BusDeviceParamRepository;
import org.rcisoft.business.manage.entity.BusDeviceParam;
import org.rcisoft.business.manage.service.BusDeviceParamService;
import org.rcisoft.business.manage.vo.BusDeviceParamList;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-2 16:09:29.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusDeviceParamServiceImpl implements BusDeviceParamService {

    @Autowired
    private BusDeviceParamRepository busDeviceParamRepository;


    /**
    * 保存 busDeviceParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusDeviceParamList busDeviceParamList){
        int line = 0;
        List<BusDeviceParam> list = busDeviceParamList.getBusDeviceParamList();
        for(BusDeviceParam bp : list){
            bp.setId(UUID.randomUUID().toString().replace("-",""));
            line = busDeviceParamRepository.insertSelective(bp);
        }
        return new PersistModel(line,"");
    }

    /**
    * 删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devNum){
        int line = 0;
        String message = "";
        Example example = new Example(BusDeviceParam.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devNum",devNum);
        Map<String,Object> map = new HashMap<>();
        map.put("dev_num",devNum);
        if(busDeviceParamRepository.queryLibraryParamByDev(map)==null || busDeviceParamRepository.queryLibraryParamByDev(map).size()<1){
            line = busDeviceParamRepository.deleteByExample(example);
        }else{
            message = "关联参数库模板不为空，无法操作";
        }
        return new PersistModel(line,message);
    }

    /**
    * 修改 busDeviceParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusDeviceParamList busDeviceParamList){
        int line = 0;
        String message = "";
        PersistModel pm = delete(busDeviceParamList.getBusDeviceParamList().get(0).getDevNum());
        if(pm.isSuccessBySinglePersist()){
            return save(busDeviceParamList);
        }else{
            message = "关联参数库模板不为空，无法操作";
        }
        return new PersistModel(line,message);
    }

    /**
    * 根据设备编号查询参数模板
    * @return
    */
    public List<Map<String,Object>> findByDev(String devNum,String proid){
        Map<String,Object> map = new HashMap<>();
        map.put("dev_num",devNum);
        map.put("pro_id",proid);
        return busDeviceParamRepository.detail(map);
    }


}
