package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.manage.dao.BusLibraryParamRepository;
import org.rcisoft.business.manage.entity.BusLibraryParam;
import org.rcisoft.business.manage.service.BusLibraryParamService;
import org.rcisoft.business.manage.vo.BusLibraryParamList;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.core.util.UserUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-2 16:09:30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusLibraryParamServiceImpl implements BusLibraryParamService {

    @Autowired
    private BusLibraryParamRepository busLibraryParamRepository;


    /**
    * 保存 busLibraryParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusLibraryParamList busLibraryParamList){
        int line = 0;
        List<BusLibraryParam> busLibraryParams = busLibraryParamList.getBusLibraryParamList();
        for(BusLibraryParam bp : busLibraryParams){
            bp.setId(UUID.randomUUID().toString().replace("-",""));
            line  = busLibraryParamRepository.insertSelective(bp);
        }
        return new PersistModel(line);
    }

    /**
    * 删除
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel delete(String devNum){
        int line = 0;
        String message = "";
        Example example = new Example(BusLibraryParam.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("devNum",devNum);
        Map<String,Object> map = new HashMap<>();
        map.put("dev_num",devNum);
        if(busLibraryParamRepository.queryParamLibraryByDev(map)==null || busLibraryParamRepository.queryParamLibraryByDev(map).size()<1){
            line = busLibraryParamRepository.deleteByExample(example);
        }else{
            message = "设备参数库不为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    /**
    * 修改 busLibraryParam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusLibraryParamList busLibraryParamList){
        String message = "";
        PersistModel pm = delete(busLibraryParamList.getBusLibraryParamList().get(0).getDevNum());
        if(pm.isSuccessBySinglePersist()){
            return save(busLibraryParamList);
        }else{
            message = "设备参数库不为空，无法更新";
        }
        return new PersistModel(0,message);
    }

    @Override
    public List<Map<String, Object>> queryLibraryParamByDev(String devNum) {
        return busLibraryParamRepository.queryLibraryParamByDev(devNum);
    }


}
