package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.manage.entity.BusFactory;
import org.rcisoft.business.manage.service.BusProjectAreaService;
import org.rcisoft.business.manage.vo.BusProjectAreaList;
import org.rcisoft.business.overview.dao.BusProjectAreaRepository;
import org.rcisoft.business.overview.entity.BusProjectArea;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import redis.clients.jedis.Transaction;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.Map;

/**
 * Created by  on 2018-5-16 14:34:55.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusProjectAreaServiceImpl implements BusProjectAreaService {

    @Autowired
    private BusProjectAreaRepository busProjectAreaRepository;


    /**
    * 保存 busProjectArea
     * 新增项目时添加
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusProjectAreaList busProjectAreaList){
        int line = 0;
        List<BusProjectArea> list  = busProjectAreaList.getBusProjectAreaList();
        for(BusProjectArea busProjectArea : list){
            line = busProjectAreaRepository.insertSelective(busProjectArea);
        }
        return new PersistModel(line);
    }

    /**
    * 修改 busProjectArea
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusProjectAreaList busProjectAreaList){
        int line = 1;
        String message = "";
        List<BusProjectArea> list  = busProjectAreaList.getBusProjectAreaList();
        for(BusProjectArea busProjectArea : list){
            Example example = new Example(BusProjectArea.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("id",busProjectArea.getId());
            if(busProjectArea.getId()!=null && !busProjectArea.getId().equals("")){
                line = busProjectAreaRepository.updateByExampleSelective(busProjectArea,example);
            }else{
                line = 0;
                message = "ID为空，更新失败";
                return new PersistModel(line,message);
            }
        }
        return new PersistModel(line,message);
    }

    @Override
    public PersistModel delete(String proId) {
        int line = 0;
        String message = "";
        Example example = new Example(BusProjectArea.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId",proId);
        if(proId!=null && !proId.equals("")){
            line = busProjectAreaRepository.deleteByExample(example);
        }else{
            line = 0;
            message = "项目ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    @Override
    public List<Map<String, Object>> queryBusProjectArea(String proId) {
        return busProjectAreaRepository.queryBusProjectArea(proId);
    }

}
