package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.manage.entity.BusProject;
import org.rcisoft.business.manage.service.BusProjectAreaService;
import org.rcisoft.business.manage.service.BusProjectService;
import org.rcisoft.business.manage.service.EnergyPriceService;
import org.rcisoft.business.manage.vo.BusProjectAreaList;
import org.rcisoft.business.manage.vo.EnergyPriceList;
import org.rcisoft.business.manage.vo.EnergyPriceVo;
import org.rcisoft.business.overview.dao.BusProjectRepository;
import org.rcisoft.business.overview.entity.BusProjectArea;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.manage.dao.ProjectRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.*;

/**
 * Created by  on 2018-5-2 16:09:30.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusProjectServiceImpl implements BusProjectService {

    @Autowired
    private ProjectRepository projectRepository;
    @Autowired
    private BusProjectRepository busProjectRepository;
    @Autowired
    private BusProjectAreaService busProjectAreaService;
    @Autowired
    private EnergyPriceService energyPriceService;


    /**
    * 保存 busProject
    * @param busProject
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusProject busProject){
        Integer line = 1;
        String message = "";
        busProject.setProId(busProject.getJwnum());
        BusProject bus = new BusProject();
        bus.setProId(busProject.getProId());
        if(findByParam(bus)!=null && findByParam(bus).size()>0){
            line = 0;
            message = "网关编号重复，项目新增失败";
            return new PersistModel(line,message);
        }
        line = projectRepository.insertSelective(busProject);
        //新增项目成功后，添加项目能耗标准、24小时的能耗价格
        if(line>0){
            //能耗标准
            BusProjectAreaList busProjectAreaList = new BusProjectAreaList();
            List<BusProjectArea> list = new ArrayList<BusProjectArea>();
            BusProjectArea a1 = new BusProjectArea();
            a1.setId(UUID.randomUUID().toString().replace("-",""));
            a1.setProId(busProject.getJwnum());
            a1.setType("1");
            list.add(a1);
            BusProjectArea a2 = new BusProjectArea();
            a2.setId(UUID.randomUUID().toString().replace("-",""));
            a2.setProId(busProject.getJwnum());
            a2.setType("2");
            list.add(a2);
            BusProjectArea a3 = new BusProjectArea();
            a3.setId(UUID.randomUUID().toString().replace("-",""));
            a3.setProId(busProject.getJwnum());
            a3.setType("3");
            list.add(a3);
            busProjectAreaList.setBusProjectAreaList(list);
            PersistModel areaP =  busProjectAreaService.save(busProjectAreaList);
            //能耗价格
            EnergyPriceList energyPriceList = new EnergyPriceList();
            List<EnergyPriceVo> plist = new ArrayList<>();
            for(int i=0;i<24;i++){
                EnergyPriceVo ep_water = new EnergyPriceVo();
                ep_water.setId(UUID.randomUUID().toString().replace("-",""));
                ep_water.setProId(busProject.getJwnum());
                ep_water.setPriceHour(String.valueOf(i));
                ep_water.setPriceTp("1");
                EnergyPriceVo ep_power = new EnergyPriceVo();
                ep_power.setId(UUID.randomUUID().toString().replace("-",""));
                ep_power.setProId(busProject.getJwnum());
                ep_power.setPriceHour(String.valueOf(i));
                ep_power.setPriceTp("2");
                EnergyPriceVo ep_gas = new EnergyPriceVo();
                ep_gas.setId(UUID.randomUUID().toString().replace("-",""));
                ep_gas.setProId(busProject.getJwnum());
                ep_gas.setPriceHour(String.valueOf(i));
                ep_gas.setPriceTp("3");
                plist.add(ep_water);
                plist.add(ep_power);
                plist.add(ep_gas);
            }
            energyPriceList.setEnergyPriceList(plist);
            PersistModel priceP =  energyPriceService.save(energyPriceList);
            if(!areaP.isSuccessBySinglePersist()){
                return areaP;
            }
            if(!priceP.isSuccessBySinglePersist()){
                return priceP;
            }
        }
        return new PersistModel(line);
    }


    /**
    * 修改 busProject
    * @param busProject
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusProject busProject){
        int line = 0;
        String message = "";
        Example example = new Example(BusProject.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId", busProject.getProId());
        if(busProject.getProId()!=null && !busProject.getProId().equals("")){
            line = projectRepository.updateByExampleSelective(busProject,example);
        }else{
            message = "ID为空，更新失败";
        }

        return new PersistModel(line,message);
    }

    @Transactional
    @Override
    public PersistModel delete(BusProject busProject) {
        int line = 0;
        String message = "";
        Map<String,Object> map = new HashMap<String,Object>();
        map.put("pro_id",busProject.getProId());
        if(busProject.getProId()!=null && !busProject.getProId().equals("")){
            //项目下没有关联的设备，可以删除
            if(projectRepository.queryDeivceByProject(map)==null || projectRepository.queryDeivceByProject(map).size()<1){
                Example example = new Example(BusProject.class);
                Example.Criteria criteria = example.createCriteria();
                criteria.andEqualTo("proId", busProject.getProId());
                line = projectRepository.deleteByExample(example);
            }else{
                message = "项目下有关联设备，无法删除";
            }
        }else{
            message = "ID为空，删除失败";
        }
        if(line>0){
            PersistModel areaP = busProjectAreaService.delete(busProject.getProId());
            PersistModel priceP = energyPriceService.delete(busProject.getProId());
            if(!areaP.isSuccessBySinglePersist()){
                return areaP;
            }
            if(!priceP.isSuccessBySinglePersist()){
                return priceP;
            }
        }
        return new PersistModel(line,message);
    }

    /**
    * 根据参数查询 busProject
    * @return
    */
    public List<Map<String,Object>> findByParam(BusProject bus){
        return projectRepository.queryBusProjects(bus);
    }

    /**
    * 分页查询 busProject
    * @param busProject
    * @return
    */
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusProject> paginationUtility,
        BusProject busProject){
        return projectRepository.queryBusProjects(null);
    }

    @Override
    public List<Map<String, Object>> queryBusProjectByUser(Map<String, Object> map) {
        return projectRepository.queryBusProjectByUser(map);
    }

}
