package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.evaluate.team.dao.BusTeamRepository;
import org.rcisoft.business.evaluate.team.entity.BusTeam;
import org.rcisoft.business.manage.service.BusTeamService;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.List;
import java.util.Map;
import java.util.UUID;

/**
 * Created by  on 2018-5-16 14:34:55.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class BusTeamServiceImpl implements BusTeamService {

    @Autowired
    private BusTeamRepository busTeamRepository;


    /**
    * 保存 busTeam
    * @param busTeam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(BusTeam busTeam){
        //增加操作
        busTeam.setTeamId(UUID.randomUUID().toString().replace("-",""));
        int line = busTeamRepository.insertSelective(busTeam);
        return new PersistModel(line);
    }

    /**
    * 修改 busTeam
    * @param busTeam
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(BusTeam busTeam){
        Integer line = 0;
        String message = "";
        Example example = new Example(BusTeam.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("teamId",busTeam.getTeamId());
        if(busTeam.getTeamId()!=null && !busTeam.getTeamId().equals("")){
            line = busTeamRepository.updateByExampleSelective(busTeam,example);
        }else{
            message = "ID为空，更新失败";
        }
        return new PersistModel(line,message);
    }

    @Override
    public PersistModel delete(String id) {
        Integer line = 0;
        String message = "";
        Example example = new Example(BusTeam.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("teamId",id);
        if(id!=null && !id.equals("")){
            line = busTeamRepository.deleteByExample(example);
        }else{
            message = "ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    /**
    * 分页查询 busTeam
    * @return
    */
    public List<Map<String,Object>> findAllByPagination(PageUtil<BusTeam> paginationUtility,
                                                        String teamType){
        return busTeamRepository.queryBusTeams(teamType);
    }

}
