package org.rcisoft.business.manage.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.rcisoft.business.manage.service.EnergyPriceService;
import org.rcisoft.business.manage.vo.EnergyPriceList;
import org.rcisoft.business.manage.vo.EnergyPriceVo;
import org.rcisoft.business.overview.dao.EnergyPriceRepository;
import org.rcisoft.business.overview.entity.EnergyPrice;
import org.rcisoft.core.model.PersistModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;
import tk.mybatis.mapper.entity.Example;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Created by  on 2018-5-16 14:34:55.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class EnergyPriceServiceImpl implements EnergyPriceService {

    @Autowired
    private EnergyPriceRepository energyPriceRepository;


    /**
     * 批量新增，新建项目时执行
     * @param energyPriceList
     * @return
     */
    @Transactional
    @Override
    public PersistModel save(EnergyPriceList energyPriceList) {
        int line = 0;
        List<EnergyPriceVo> list  = energyPriceList.getEnergyPriceList();
        for(EnergyPriceVo energyPriceVo : list){
            line = energyPriceRepository.insertSelective(energyPriceVo);
        }
        return new PersistModel(line);
    }

    @Override
    public PersistModel delete(String proId) {
        int line = 0;
        String message = "";
        Example example = new Example(EnergyPriceVo.class);
        Example.Criteria criteria = example.createCriteria();
        criteria.andEqualTo("proId",proId);
        if(proId!=null && !proId.equals("")){
            line = energyPriceRepository.deleteByExample(example);
        }else{
            line = 0;
            message = "项目ID为空，删除失败";
        }
        return new PersistModel(line,message);
    }

    @Transactional
    @Override
    public PersistModel merge(EnergyPriceList energyPriceList) {
        int line = 1;
        String message = "";
        List<EnergyPriceVo> list  = energyPriceList.getEnergyPriceList();
        for(EnergyPriceVo energyPriceVo : list){
            Example example = new Example(EnergyPriceVo.class);
            Example.Criteria criteria = example.createCriteria();
            criteria.andEqualTo("id",energyPriceVo.getId());
            if(energyPriceVo.getId()!=null && !energyPriceVo.getId().equals("")){
                line = energyPriceRepository.updateByExampleSelective(energyPriceVo,example);
            }else{
                message = "ID为空，更新失败";
                line = 0;
                return new PersistModel(line,message);
            }
        }
        return new PersistModel(line,message);
    }

    @Override
    public Map<String, Object> queryEnergyPrice(String proId) {
        Map<String, Object> result = new HashMap<>();
        List<Map<String, Object>> list = energyPriceRepository.queryEnergyPrice(proId);
        Map<String, Object> water_map = new HashMap<>();
        Map<String, Object> power_map = new HashMap<>();
        Map<String, Object> gas_map = new HashMap<>();
        for(Map<String, Object> mp : list){
            if(mp.get("P_TP")!=null && mp.get("P_TP").toString().equals("1")){
                water_map.put(mp.get("P_HOUR").toString(),mp.get("P_PRICE"));
            }else if(mp.get("P_TP")!=null && mp.get("P_TP").toString().equals("2")){
                power_map.put(mp.get("P_HOUR").toString(),mp.get("P_PRICE"));
            }else if(mp.get("P_TP")!=null && mp.get("P_TP").toString().equals("3")){
                gas_map.put(mp.get("P_HOUR").toString(),mp.get("P_PRICE"));
            }
        }
        result.put("water",water_map);
        result.put("power",power_map);
        result.put("gas",gas_map);
        return result;
    }
}
