package org.rcisoft.business.overview.dao;

import org.rcisoft.business.overview.entity.EnergyEmissionReal;
import org.rcisoft.core.base.BaseMapper;
import org.apache.ibatis.annotations.ResultMap;
import org.apache.ibatis.annotations.Select;
import org.springframework.stereotype.Repository;


import java.util.List;

/**
* Created with  on 2018-4-14 17:41:53.
*/
@Repository
public interface EnergyEmissionRealRepository extends BaseMapper<EnergyEmissionReal> {

    /**
     * 根据年月日查询碳排放量
     * @param energyEmissionReal proId：项目主键 year：年 mon：月 day：日
     * @return
     */
    @Select("<script>select * from energy_emission_real where PRO_ID=#{proId} and `YEAR`=#{year} and MON=#{mon} and `DAY`=#{day}</script>")
    @ResultMap(value = "BaseResultMap" )
    EnergyEmissionReal queryEnergyEmissionReal(EnergyEmissionReal energyEmissionReal);

    /**
     * 按照月份或年份统计
     * @param energyEmissionReal proId,year,(mon)
     * @return
     */
    @Select("<script>select sum(e.EMISSION) as EMISSION from energy_emission_real e where e.PRO_ID=#{proId} and e.`YEAR`=#{year}<if test=\"mon != null\"> and e.MON=#{mon}</if></script>")
    @ResultMap(value = "BaseResultMap" )
    EnergyEmissionReal statEnergyEmissionReal(EnergyEmissionReal energyEmissionReal);

    /**
     * 月 碳排放
     * @param energyEmissionReal
     * @return
     */
    @Select("<script>select e.MON,sum(e.EMISSION) as EMISSION from energy_emission_real e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} group by e.MON asc</script>")
    @ResultMap(value = "BaseResultMap" )
    List<EnergyEmissionReal> statEmissionMonth(EnergyEmissionReal energyEmissionReal);

    /**
     * 日 碳排放
     * @param energyEmissionReal
     * @return
     */
    @Select("<script>select e.`DAY`,sum(e.EMISSION) as EMISSION from energy_emission_real e where e.PRO_ID=#{proId} and e.`YEAR`=#{year} and e.MON=#{mon} group by e.`DAY` asc</script>")
    @ResultMap(value = "BaseResultMap" )
    List<EnergyEmissionReal> statEmissionDay(EnergyEmissionReal energyEmissionReal);

}

