package org.rcisoft.business.overview.service;

import org.rcisoft.business.manage.vo.EnergyPriceVo;
import org.rcisoft.business.overview.entity.EnergyEmissionTotal;
import org.rcisoft.business.overview.vo.EnergyCountMVo;
import org.rcisoft.core.model.PersistModel;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

/**
 * Created by JiChao on 2018/4/11.
 * 概览
 */
public interface OverViewService {

    /**
     * 根据项目id查询价格
     * @param proId
     * @return
     */
    EnergyPriceVo getPrice(String proId);

    /**
     * 用量和费用统计，当日或当月
     * @param proId
     * @return
     */
    Map<String, Object> countAmountAndPrice(String proId);

    /**
     * 运行费用统计
     * @param proId
     * @param type
     * @return
     */
    List<EnergyCountMVo> countDailyPrice(String proId, Integer type, Integer date);

    /**
     * 耗能拆分统计
     * @param proId
     * @param type
     * @return
     */
    List<EnergyCountMVo> countEnergySplit(String proId, Integer type, Integer date);

    /**
     * 运行费用排名
     * @param proId
     * @param type
     * @param date
     * @return
     */
    Integer getPriceRank(String proId, Integer type, Integer date);

    /**
     * 建筑单位面积用量标准
     * @param proId
     * @return
     */
    Map<String, Object> getStandard(String proId);

    /**
     * 建筑单位面积用量年度，三年的数据
     * @param proId
     * @return
     */
    Map<String, Object> getUnitArea(String proId);

    /**
     * 用能比较
     * @param proId
     * @param year
     * @param month
     * @param type
     * @param cType
     * @return
     */
    Map<String, Object> getEnergyCompare(String proId, String year, String month, Integer type, Integer cType);

    /**
     * 每月统计能量
     * @return
     */
    PersistModel statEnergyMonthly();

    /**
     * 查询8条碳排放量数据，当日、日同比、环比；当月、月同比、环比；当年、年同比
     * @param proId
     * @return
     */
    Map<String, BigDecimal> getEmission(String proId);

    /**
     * 查询累计减排
     * @param proId
     * @return
     */
    EnergyEmissionTotal getEmissionStat(String proId);

    /**
     * 月 碳排放
     * @param proId
     * @param year
     * @return
     */
    List<BigDecimal> statEmissionMonth(String proId, int year);

    /**
     * 日 碳排放
     * @param proId
     * @param year
     * @param month
     * @return
     */
    List<BigDecimal> statEmissionDay(String proId, int year, int month);

    /**
     * 统计昨天排放量，更新累计排放量
     * @return
     */
    PersistModel statEmission();

    /**
     * 年用能计划
     * @param proId
     * @param year
     * @return
     */
    Map<String, Object> statEnergyPlanYear(String proId, String year);

    /**
     * 月用能计划
     * @param proId
     * @param year
     * @param month
     * @return
     */
    Map<String, Object> statEnergyPlanMonth( String proId,  String year,  String month);

    /**
     * 日用能计划
     * @param proId
     * @param year
     * @param month
     * @param day
     * @return
     */
    Map<String, Object> statEnergyPlanDay( String proId,  String year,  String month,  String day);

}
