package org.rcisoft.business.system.user.service.impl;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.rcisoft.business.system.user.dao.RoleRepository;
import org.rcisoft.business.system.user.dao.UserRepository;
import org.rcisoft.business.system.user.entity.SysRole;
import org.rcisoft.business.system.user.entity.SysUser;
import org.rcisoft.core.util.JwtUserFactory;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.core.userdetails.UserDetailsService;
import org.springframework.security.core.userdetails.UsernameNotFoundException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.util.Collections;
import java.util.List;

/**
 * @Author: GaoLiWei
 * @Date: Created in 11:112018/5/8
 */
@Service
@Transactional(readOnly = true, propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class UserDetailsServiceImpl implements UserDetailsService {

    @Autowired
    private UserRepository userRepository;
    @Autowired
    private RoleRepository roleRepository;

    @Override
    public UserDetails loadUserByUsername(String username) throws UsernameNotFoundException {
        //调用持久层从数据库获取用户信息

        List<SysUser> sysUserList = userRepository.listByUserName(username);

        if (sysUserList == null || sysUserList.size() == 0) {
            throw new UsernameNotFoundException("用户名不存在");
        }
        SysUser sysUser = sysUserList.get(0);
        /*根据用户查询用户权限*/
        List<SysRole> roles = roleRepository.listRoleByUserId(sysUser.getUserId());
        if (CollectionUtils.isEmpty(roles)) {
            roles = Collections.emptyList();
        }
        sysUser.setRoleList(roles);
        return JwtUserFactory.create(sysUser);
    }
}
