package org.rcisoft.business.totaloriginal.service.impl;

import org.rcisoft.business.totaloriginal.dao.TotalSensorRepository;
import org.rcisoft.business.totaloriginal.entity.TotalSensor;
import org.rcisoft.core.util.UserUtil;
import org.rcisoft.core.aop.PageUtil;
import org.rcisoft.core.model.PersistModel;
import org.rcisoft.business.totaloriginal.dao.TotalOriginalRepository;
import org.rcisoft.business.totaloriginal.entity.TotalOriginal;
import org.rcisoft.business.totaloriginal.service.TotalOriginalService;


import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Isolation;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;


import java.util.HashMap;
import java.util.List;
import java.util.Map;

import lombok.extern.slf4j.Slf4j;

/**
 * Created by  on 2018-4-12 15:48:19.
 */
@Service
@Transactional(readOnly = true,propagation = Propagation.NOT_SUPPORTED)
@Slf4j
public class TotalOriginalServiceImpl implements TotalOriginalService {

    @Autowired
    private TotalOriginalRepository totalOriginalRepository;
    @Autowired
    private TotalSensorRepository totalSensorRepository;


    /**
    * 保存 totalOriginal
    * @param totalOriginal
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel save(TotalOriginal totalOriginal){
        //增加操作
        int line = totalOriginalRepository.insertSelective(totalOriginal);
        return new PersistModel(line);
    }

    /**
    * 逻辑删除
    * @param totalOriginal
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel remove(TotalOriginal totalOriginal){
        int line = totalOriginalRepository.logicalDelete(totalOriginal);

        return new PersistModel(line);
    }

    /**
    * 修改 totalOriginal
    * @param totalOriginal
    * @return
    */
    @Transactional(propagation = Propagation.REQUIRED,isolation = Isolation.DEFAULT)
    @Override
    public PersistModel merge(TotalOriginal totalOriginal){
        int line = totalOriginalRepository.updateByPrimaryKeySelective(totalOriginal);
        return new PersistModel(line);
    }

    /**
    * 根据id查询 totalOriginal
    * @param id
    * @return
    */
    public TotalOriginal findById(int id){
        return totalOriginalRepository.selectByPrimaryKey(id);
    }

    /**
    * 分页查询 totalOriginal
    * @param totalOriginal
    * @return
    */
    public List<TotalOriginal> findAllByPagination(PageUtil<TotalOriginal> paginationUtility,
        TotalOriginal totalOriginal){
        return totalOriginalRepository.queryTotalOriginals(totalOriginal);
    }

    @Override
    public PersistModel saveSensor(TotalSensor totalsensor) {
        int line = totalSensorRepository.insertSelective(totalsensor);
        return new PersistModel(line);
    }

    @Override
    public Map<String, Object> queryBusParamRefer(Map<String, Object> map) {
        List<Map<String, Object>> list = totalOriginalRepository.queryBusParamRefer(map);
        Map<String, Object> referMap = new HashMap<String, Object>();
        for(Map<String, Object> tmap : list){
            referMap.put(tmap.get("DEV_NUM").toString(),tmap.get("OWN_PARAM").toString()+","+tmap.get("OTHER_PARAM").toString());
        }
        return referMap;
    }

    @Override
    public String queryDevByPro(String jwnum) {
        Map<String, Object> map = new HashMap<>();
        map.put("jwnum",jwnum);
        List<Map<String, Object>> list = totalOriginalRepository.queryDevByPro(map);
        String devStr = "";
        for (Map<String, Object> tmap : list){
            devStr += tmap.get("DEV_NUM");
        }
        return devStr;
    }

}
