package org.rcisoft.core.model;

import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.Data;
import org.springframework.security.core.GrantedAuthority;
import org.springframework.security.core.userdetails.UserDetails;

import java.util.Collection;

/**
 * Created by lcy on 17/11/21.
 */
@Data
public class JwtUser implements UserDetails{


    private static final long serialVersionUID = -4070913768662945949L;

    private final String businessId;
    private final String username;
    private final String password;

    /** 权限类.*/
    private final Collection<? extends GrantedAuthority> authorities;


    /**
     * 在createJwtFactory里注入
     */
    public JwtUser(String id,
                   String username,
                   String password,
                   Collection<? extends GrantedAuthority> authorities) {
        this.businessId = id;
        this.username = username;
        this.password = password;
        this.authorities = authorities;
    }

    public String getBusinessId() {
        return businessId;
    }

    @Override
    public Collection<? extends GrantedAuthority> getAuthorities() {
        return authorities;
    }

    @Override
    @JsonIgnore
    public String getPassword() {
        return password;
    }

    @Override
    public String getUsername() {
        return username;
    }

    @Override
    @JsonIgnore
    public boolean isAccountNonExpired() {
        return true;
    }
    @Override
    @JsonIgnore
    public boolean isAccountNonLocked() {
        return true;
    }
    @Override
    @JsonIgnore
    public boolean isCredentialsNonExpired() {
        return true;
    }
    @Override
    @JsonIgnore
    public boolean isEnabled() {
        return true;
    }
}
